@@ -48,12 +48,11 @@ static ulonglong system_variable_hash_version= 0;
   Return variable name and length for hashing of variables.
 */
 
-static const uchar *get_sys_var_length(const void *var_, size_t *length,
-                                       my_bool)
+static uchar *get_sys_var_length(const sys_var *var, size_t *length,
+                                 my_bool first)
 {
-  auto var= static_cast<const sys_var *>(var_);
   *length= var->name.length;
-  return reinterpret_cast<const uchar *>(var->name.str);
+  return (uchar*) var->name.str;
 }
 
 sys_var_chain all_sys_vars = { NULL, NULL };
@@ -66,8 +65,8 @@ int sys_var_init()
   DBUG_ASSERT(system_charset_info != NULL);
 
   if (my_hash_init(PSI_INSTRUMENT_ME, &system_variable_hash,
-                   system_charset_info, 700, 0, 0, get_sys_var_length, 0,
-                   HASH_UNIQUE))
+                   Lex_ident_sys_var::charset_info(), 700, 0,
+                   0, (my_hash_get_key) get_sys_var_length, 0, HASH_UNIQUE))
     goto error;
 
   if (mysql_add_sys_var_chain(all_sys_vars.first))
@@ -155,8 +154,7 @@ sys_var::sys_var(sys_var_chain *chain, const char *name_arg,
                  const char *substitute) :
   next(0), binlog_status(binlog_status_arg), value_origin(COMPILE_TIME),
   flags(flags_arg), show_val_type(show_val_type_arg),
-  guard(lock), offset(off), on_check(on_check_func), on_update(on_update_func),
-  deprecation_substitute(substitute)
+  guard(lock), offset(off), on_check(on_check_func), on_update(on_update_func)
 {
   /*
     There is a limitation in handle_options() related to short options:
@@ -172,6 +170,8 @@ sys_var::sys_var(sys_var_chain *chain, const char *name_arg,
   name.str= name_arg;     // ER_NO_DEFAULT relies on 0-termination of name_arg
   name.length= strlen(name_arg);                // and so does this.
   DBUG_ASSERT(name.length <= NAME_CHAR_LEN);
+  DBUG_ASSERT(!comment || !comment[0] || comment[strlen(comment)-1] != '.');
+  DBUG_ASSERT(!comment || !comment[0] || comment[strlen(comment)-1] != ' ');
 
   bzero(&option, sizeof(option));
   option.name= name_arg;
@@ -182,6 +182,7 @@ sys_var::sys_var(sys_var_chain *chain, const char *name_arg,
   option.def_value= def_val;
   option.app_type= this;
   option.var_type= flags & AUTO_SET ? GET_AUTO : 0;
+  option.deprecation_substitute= substitute;
 
   if (chain->last)
     chain->last->next= this;
@@ -419,14 +420,28 @@ double sys_var::val_real(bool *is_null,
   return ret;
 }
 
+/* Marker if the variable is deleted instead of depricated */
+const char *UNUSED_HELP="Unused";
 
 void sys_var::do_deprecated_warning(THD *thd)
 {
-  if (deprecation_substitute != NULL)
+  if (option.deprecation_substitute != NULL)
   {
     char buf1[NAME_CHAR_LEN + 3];
     strxnmov(buf1, sizeof(buf1)-1, "@@", name.str, 0);
-    warn_deprecated<999999>(thd, buf1, deprecation_substitute);
+
+    if (option.comment == UNUSED_HELP ||
+        strcmp(option.comment, UNUSED_HELP) == 0)
+      my_error(ER_VARIABLE_IGNORED, MYF(ME_WARNING), buf1);
+    else
+    {
+      char buf2[NAME_CHAR_LEN + 3];
+      if (!IS_DEPRECATED_NO_REPLACEMENT(option.deprecation_substitute))
+        strxnmov(buf2, sizeof(buf2)-1, "@@", option.deprecation_substitute, 0);
+      else
+        buf2[0]= 0;
+      warn_deprecated<999999>(thd, buf1, buf2);
+    }
   }
 }
 
@@ -501,31 +516,31 @@ bool throw_bounds_warning(THD *thd, const char *name, bool fixed, double v)
 
 typedef struct old_names_map_st
 {
-  const char *old_name;
+  const Lex_ident_charset old_name;
   const char *new_name;
 } my_old_conv;
 
 static my_old_conv old_conv[]=
 {
-  {     "cp1251_koi8"           ,       "cp1251"        },
-  {     "cp1250_latin2"         ,       "cp1250"        },
-  {     "kam_latin2"            ,       "keybcs2"       },
-  {     "mac_latin2"            ,       "MacRoman"      },
-  {     "macce_latin2"          ,       "MacCE"         },
-  {     "pc2_latin2"            ,       "pclatin2"      },
-  {     "vga_latin2"            ,       "pclatin1"      },
-  {     "koi8_cp1251"           ,       "koi8r"         },
-  {     "win1251ukr_koi8_ukr"   ,       "win1251ukr"    },
-  {     "koi8_ukr_win1251ukr"   ,       "koi8u"         },
-  {     NULL                    ,       NULL            }
+  {     "cp1251_koi8"_Lex_ident_charset          ,       "cp1251"        },
+  {     "cp1250_latin2"_Lex_ident_charset        ,       "cp1250"        },
+  {     "kam_latin2"_Lex_ident_charset           ,       "keybcs2"       },
+  {     "mac_latin2"_Lex_ident_charset           ,       "MacRoman"      },
+  {     "macce_latin2"_Lex_ident_charset         ,       "MacCE"         },
+  {     "pc2_latin2"_Lex_ident_charset           ,       "pclatin2"      },
+  {     "vga_latin2"_Lex_ident_charset           ,       "pclatin1"      },
+  {     "koi8_cp1251"_Lex_ident_charset          ,       "koi8r"         },
+  {     "win1251ukr_koi8_ukr"_Lex_ident_charset  ,       "win1251ukr"    },
+  {     "koi8_ukr_win1251ukr"_Lex_ident_charset  ,       "koi8u"         },
+  {     Lex_ident_charset()                      ,       NULL            }
 };
 
-CHARSET_INFO *get_old_charset_by_name(const char *name)
+CHARSET_INFO *get_old_charset_by_name(const LEX_CSTRING &name)
 {
   my_old_conv *conv;
-  for (conv= old_conv; conv->old_name; conv++)
+  for (conv= old_conv; conv->old_name.str; conv++)
   {
-    if (!my_strcasecmp(&my_charset_latin1, name, conv->old_name))
+    if (conv->old_name.streq(name))
       return get_charset_by_csname(conv->new_name, MY_CS_PRIMARY, MYF(0));
   }
   return NULL;
@@ -603,10 +618,8 @@ int mysql_del_sys_var_chain(sys_var *first)
 }
 
 
-static int show_cmp(const void *a_, const void *b_)
+static int show_cmp(SHOW_VAR *a, SHOW_VAR *b)
 {
-  const SHOW_VAR *a= static_cast<const SHOW_VAR *>(a_);
-  const SHOW_VAR *b= static_cast<const SHOW_VAR *>(b_);
   return strcmp(a->name, b->name);
 }
 
@@ -782,7 +795,6 @@ int set_var::check(THD *thd)
   if (!value)
     return 0;
 
-  thd->where= THD_WHERE::SET_LIST;
   if (value->fix_fields_if_needed_for_scalar(thd, &value))
     return -1;
   if (var->check_update_type(value))
@@ -889,7 +901,6 @@ int set_var_user::check(THD *thd)
     Item_func_set_user_var can't substitute something else on its place =>
     0 can be passed as last argument (reference on item)
   */
-  thd->where= THD_WHERE::SET_LIST;
   return (user_var_item->fix_fields(thd, (Item**) 0) ||
           user_var_item->check(0)) ? -1 : 0;
 }
@@ -965,7 +976,7 @@ int set_var_password::update(THD *thd)
 int set_var_role::check(THD *thd)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
-  int status= acl_check_setrole(thd, role.str, &access);
+  int status= acl_check_setrole(thd, role, &access);
   return status;
 #else
   return 0;
@@ -975,7 +986,7 @@ int set_var_role::check(THD *thd)
 int set_var_role::update(THD *thd)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
-  int res= acl_setrole(thd, role.str, access);
+  int res= acl_setrole(thd, role, access);
   if (!res)
     thd->session_tracker.state_change.mark_as_changed(thd);
   return res;
@@ -992,17 +1003,17 @@ int set_var_default_role::check(THD *thd)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   real_user= get_current_user(thd, user);
-  real_role= role.str;
+  real_role= role;
   if (role.str == current_role.str)
   {
     if (!thd->security_ctx->priv_role[0])
-      real_role= "NONE";
+      real_role= "NONE"_LEX_CSTRING;
     else
-      real_role= thd->security_ctx->priv_role;
+      real_role= Lex_cstring_strlen(thd->security_ctx->priv_role);
   }
 
-  return acl_check_set_default_role(thd, real_user->host.str,
-                                    real_user->user.str, real_role);
+  return acl_check_set_default_role(thd, real_user->host,
+                                    real_user->user, real_role);
 #else
   return 0;
 #endif
@@ -1013,7 +1024,7 @@ int set_var_default_role::update(THD *thd)
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   Reprepare_observer *save_reprepare_observer= thd->m_reprepare_observer;
   thd->m_reprepare_observer= 0;
-  int res= acl_set_default_role(thd, real_user->host.str, real_user->user.str,
+  int res= acl_set_default_role(thd, real_user->host, real_user->user,
                                 real_role);
   thd->m_reprepare_observer= save_reprepare_observer;
   return res;
@@ -1081,7 +1092,6 @@ static void store_var(Field *field, sys_var *var, enum_var_type scope,
 
 int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
 {
-  char name_buffer[NAME_CHAR_LEN];
   bool res= 1;
   CHARSET_INFO *scs= system_charset_info;
   StringBuffer<STRING_BUFFER_USUAL_SIZE> strbuf(scs);
@@ -1097,16 +1107,15 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
   for (uint i= 0; i < system_variable_hash.records; i++)
   {
     sys_var *var= (sys_var*) my_hash_element(&system_variable_hash, i);
-
-    strmake_buf(name_buffer, var->name.str);
-    my_caseup_str(system_charset_info, name_buffer);
+    CharBuffer<NAME_CHAR_LEN> name_buffer;
+    name_buffer.copy_caseup(scs, var->name);
 
     /* this must be done before evaluating cond */
     restore_record(tables->table, s->default_values);
-    fields[0]->store(name_buffer, strlen(name_buffer), scs);
+    fields[0]->store(name_buffer.to_lex_cstring(), scs);
 
-    if ((wild && wild_case_compare(system_charset_info, name_buffer, wild))
-        || (cond && !cond->val_bool()))
+    if ((wild && wild_case_compare(scs, name_buffer.ptr(), wild))
+        || (cond && !cond->val_int()))
       continue;
 
     mysql_mutex_lock(&LOCK_global_system_variables);
@@ -1181,7 +1190,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
 
     // VARIABLE_COMMENT
     fields[7]->store(var->option.comment, strlen(var->option.comment),
-                           scs);
+                    scs);
 
     // NUMERIC_MIN_VALUE
     // NUMERIC_MAX_VALUE
