@@ -56,11 +56,6 @@ static MEM_ROOT mem;
 static mysql_rwlock_t THR_LOCK_servers;
 static LEX_CSTRING MYSQL_SERVERS_NAME= {STRING_WITH_LEN("servers") };
 
-enum servers_fields {
-  SERVER_NAME_FIELD, HOST_FIELD, DB_FIELD, USERNAME_FIELD, PASSWORD_FIELD,
-  PORT_FIELD, SOCKET_FIELD, SCHEME_FIELD, OWNER_FIELD,
-  SERVERS_FIELDS_COUNT
-};
 
 static bool get_server_from_table_to_cache(TABLE *table);
 
@@ -86,17 +81,16 @@ static int update_server_record_in_cache(FOREIGN_SERVER *existing,
 /* utility functions */
 static void merge_server_struct(FOREIGN_SERVER *from, FOREIGN_SERVER *to);
 
-static const uchar *servers_cache_get_key(const void *server_, size_t *length,
-                                          my_bool)
+static uchar *servers_cache_get_key(FOREIGN_SERVER *server, size_t *length,
+			       my_bool not_used __attribute__((unused)))
 {
-  auto server= static_cast<const FOREIGN_SERVER *>(server_);
   DBUG_ENTER("servers_cache_get_key");
   DBUG_PRINT("info", ("server_name_length %zd server_name %s",
                       server->server_name_length,
                       server->server_name));
 
-  *length= server->server_name_length;
-  DBUG_RETURN(reinterpret_cast<const uchar *>(server->server_name));
+  *length= (uint) server->server_name_length;
+  DBUG_RETURN((uchar*) server->server_name);
 }
 
 static PSI_memory_key key_memory_servers;
@@ -238,8 +232,10 @@ bool servers_init(bool dont_read_servers_table)
     DBUG_RETURN(TRUE);
 
   /* initialise our servers cache */
-  if (my_hash_init(key_memory_servers, &servers_cache, system_charset_info, 32,
-                   0, 0, servers_cache_get_key, 0, 0))
+  if (my_hash_init(key_memory_servers, &servers_cache,
+                   Lex_ident_server::charset_info(),
+                   32, 0, 0,
+                   (my_hash_get_key) servers_cache_get_key, 0, 0))
   {
     return_val= TRUE; /* we failed, out of memory? */
     goto end;
@@ -257,7 +253,10 @@ bool servers_init(bool dont_read_servers_table)
   */
   if (!(thd=new THD(0)))
     DBUG_RETURN(TRUE);
+  thd->thread_stack= (char*) &thd;
   thd->store_globals();
+  thd->set_query_inner((char*) STRING_WITH_LEN("intern:servers_init"),
+                       default_charset_info);
   /*
     It is safe to call servers_reload() since servers_* arrays and hashes which
     will be freed there are global static objects and thus are initialized
@@ -288,7 +287,7 @@ end:
 
 static bool servers_load(THD *thd, TABLE_LIST *tables)
 {
-  TABLE *table= tables[0].table;
+  TABLE *table;
   READ_RECORD read_record_info;
   bool return_val= TRUE;
   DBUG_ENTER("servers_load");
@@ -297,8 +296,8 @@ static bool servers_load(THD *thd, TABLE_LIST *tables)
   free_root(&mem, MYF(0));
   init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE, 0, MYF(0));
 
-  table->use_all_columns();
-  if (init_read_record(&read_record_info,thd,table, NULL, NULL, 1,0, FALSE))
+  if (init_read_record(&read_record_info,thd,table=tables[0].table, NULL, NULL,
+                       1,0, FALSE))
     DBUG_RETURN(1);
   while (!(read_record_info.read_record()))
   {
@@ -315,20 +314,6 @@ end:
 }
 
 
-static bool servers_table_is_valid(TABLE *table)
-{
-  if (table->s->fields < SERVERS_FIELDS_COUNT ||
-      table->s->primary_key == MAX_KEY)
-  {
-    my_errno= 1;
-    my_error(ER_CANNOT_LOAD_FROM_TABLE_V2, MYF(0),
-             table->s->db.str, table->s->table_name.str);
-    return false;
-  }
-  return true;
-}
-
-
 /*
   Forget current servers cache and read new servers 
   from the conneciton table.
@@ -350,16 +335,17 @@ static bool servers_table_is_valid(TABLE *table)
 
 bool servers_reload(THD *thd)
 {
-  TABLE_LIST tables;
+  TABLE_LIST tables[1];
   bool return_val= TRUE;
   DBUG_ENTER("servers_reload");
 
   DBUG_PRINT("info", ("locking servers_cache"));
   mysql_rwlock_wrlock(&THR_LOCK_servers);
 
-  tables.init_one_table(&MYSQL_SCHEMA_NAME, &MYSQL_SERVERS_NAME, 0, TL_READ);
+  tables[0].init_one_table(&MYSQL_SCHEMA_NAME, &MYSQL_SERVERS_NAME, 0, TL_READ);
 
-  if (open_and_lock_tables(thd, &tables, FALSE, MYSQL_LOCK_IGNORE_TIMEOUT))
+  if (unlikely(open_and_lock_tables(thd, tables, FALSE,
+                                    MYSQL_LOCK_IGNORE_TIMEOUT)))
   {
     /*
       Execution might have been interrupted; only print the error message
@@ -372,10 +358,7 @@ bool servers_reload(THD *thd)
     goto end;
   }
 
-  if (!servers_table_is_valid(tables.table))
-    goto end;
-
-  if ((return_val= servers_load(thd, &tables)))
+  if ((return_val= servers_load(thd, tables)))
   {					// Error. Revert to old list
     /* blast, for now, we have no servers, discuss later way to preserve */
 
@@ -426,29 +409,30 @@ get_server_from_table_to_cache(TABLE *table)
   FOREIGN_SERVER *server= (FOREIGN_SERVER *)alloc_root(&mem,
                                                        sizeof(FOREIGN_SERVER));
   DBUG_ENTER("get_server_from_table_to_cache");
+  table->use_all_columns();
 
   /* get each field into the server struct ptr */
-  ptr= get_field(&mem, table->field[SERVER_NAME_FIELD]);
+  ptr= get_field(&mem, table->field[0]);
   server->server_name= ptr ? ptr : blank;
   server->server_name_length= (uint) strlen(server->server_name);
-  ptr= get_field(&mem, table->field[HOST_FIELD]);
+  ptr= get_field(&mem, table->field[1]);
   server->host= ptr ? ptr : blank;
-  ptr= get_field(&mem, table->field[DB_FIELD]);
+  ptr= get_field(&mem, table->field[2]);
   server->db= ptr ? ptr : blank;
-  ptr= get_field(&mem, table->field[USERNAME_FIELD]);
+  ptr= get_field(&mem, table->field[3]);
   server->username= ptr ? ptr : blank;
-  ptr= get_field(&mem, table->field[PASSWORD_FIELD]);
+  ptr= get_field(&mem, table->field[4]);
   server->password= ptr ? ptr : blank;
-  ptr= get_field(&mem, table->field[PORT_FIELD]);
+  ptr= get_field(&mem, table->field[5]);
   server->sport= ptr ? ptr : blank;
 
   server->port= server->sport ? atoi(server->sport) : 0;
 
-  ptr= get_field(&mem, table->field[SOCKET_FIELD]);
+  ptr= get_field(&mem, table->field[6]);
   server->socket= ptr && strlen(ptr) ? ptr : blank;
-  ptr= get_field(&mem, table->field[SCHEME_FIELD]);
+  ptr= get_field(&mem, table->field[7]);
   server->scheme= ptr ? ptr : blank;
-  ptr= get_field(&mem, table->field[OWNER_FIELD]);
+  ptr= get_field(&mem, table->field[8]);
   server->owner= ptr ? ptr : blank;
   DBUG_PRINT("info", ("server->server_name %s", server->server_name));
   DBUG_PRINT("info", ("server->host %s", server->host));
@@ -482,7 +466,7 @@ get_server_from_table_to_cache(TABLE *table)
 
   RETURN VALUES
     0  - no error
-    other - ER_ error code
+    other - error code
 */
 
 static int 
@@ -496,10 +480,8 @@ insert_server(THD *thd, FOREIGN_SERVER *server)
   tables.init_one_table(&MYSQL_SCHEMA_NAME, &MYSQL_SERVERS_NAME, 0, TL_WRITE);
 
   /* need to open before acquiring THR_LOCK_plugin or it will deadlock */
-  table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT);
-  if (!table || !servers_table_is_valid(table))
+  if (! (table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
     goto end;
-
   table->file->row_logging= 0;                  // Don't log to binary log
 
   /* insert the server into the table */
@@ -568,16 +550,15 @@ insert_server_record_into_cache(FOREIGN_SERVER *server)
     advance of insertion into the mysql.servers table
 
   RETURN VALUE
-    0 - no errors
-    >0 - ER_ error code
+    VOID
+
 */
 
-static int
+static void 
 store_server_fields(TABLE *table, FOREIGN_SERVER *server)
 {
 
   table->use_all_columns();
-
   /*
     "server" has already been prepped by prepare_server_struct_for_<>
     so, all we need to do is check if the value is set (> -1 for port)
@@ -586,43 +567,30 @@ store_server_fields(TABLE *table, FOREIGN_SERVER *server)
     have changed will be set. If an insert, then all will be set,
     even if with empty strings
   */
-  if (server->host &&
-    table->field[HOST_FIELD]->store(server->host,
-                           (uint) strlen(server->host), system_charset_info))
-    goto err;
-  if (server->db &&
-    table->field[DB_FIELD]->store(server->db,
-                           (uint) strlen(server->db), system_charset_info))
-    goto err;
-  if (server->username &&
-    table->field[USERNAME_FIELD]->store(server->username,
-                           (uint) strlen(server->username), system_charset_info))
-    goto err;
-  if (server->password &&
-    table->field[PASSWORD_FIELD]->store(server->password,
-                           (uint) strlen(server->password), system_charset_info))
-    goto err;
-  if (server->port > -1 &&
-    table->field[PORT_FIELD]->store(server->port))
-    goto err;
-  if (server->socket &&
-    table->field[SOCKET_FIELD]->store(server->socket,
-                           (uint) strlen(server->socket), system_charset_info))
-    goto err;
-  if (server->scheme &&
-    table->field[SCHEME_FIELD]->store(server->scheme,
-                           (uint) strlen(server->scheme), system_charset_info))
-    goto err;
-  if (server->owner &&
-    table->field[OWNER_FIELD]->store(server->owner,
-                           (uint) strlen(server->owner), system_charset_info))
-    goto err;
-  return 0;
+  if (server->host)
+    table->field[1]->store(server->host,
+                           (uint) strlen(server->host), system_charset_info);
+  if (server->db)
+    table->field[2]->store(server->db,
+                           (uint) strlen(server->db), system_charset_info);
+  if (server->username)
+    table->field[3]->store(server->username,
+                           (uint) strlen(server->username), system_charset_info);
+  if (server->password)
+    table->field[4]->store(server->password,
+                           (uint) strlen(server->password), system_charset_info);
+  if (server->port > -1)
+    table->field[5]->store(server->port);
 
-err:
-  THD *thd= table->in_use;
-  DBUG_ASSERT(thd->is_error());
-  return thd->get_stmt_da()->get_sql_errno();
+  if (server->socket)
+    table->field[6]->store(server->socket,
+                           (uint) strlen(server->socket), system_charset_info);
+  if (server->scheme)
+    table->field[7]->store(server->scheme,
+                           (uint) strlen(server->scheme), system_charset_info);
+  if (server->owner)
+    table->field[8]->store(server->owner,
+                           (uint) strlen(server->owner), system_charset_info);
 }
 
 /*
@@ -644,7 +612,7 @@ err:
 
   RETURN VALUE
     0 - no errors
-    >0 - ER_ error code
+    >0 - error code
 
   */
 
@@ -659,14 +627,17 @@ int insert_server_record(TABLE *table, FOREIGN_SERVER *server)
   empty_record(table);
 
   /* set the field that's the PK to the value we're looking for */
-  table->field[SERVER_NAME_FIELD]->store(server->server_name,
-                                         server->server_name_length,
-                                         system_charset_info);
+  table->field[0]->store(server->server_name,
+                         server->server_name_length,
+                         system_charset_info);
 
   /* read index until record is that specified in server_name */
-  if ((error= table->file->ha_index_read_idx_map(table->record[0], 0,
-                  (uchar *)table->field[SERVER_NAME_FIELD]->ptr,
-                  HA_WHOLE_KEY, HA_READ_KEY_EXACT)))
+  if (unlikely((error=
+                table->file->ha_index_read_idx_map(table->record[0], 0,
+                                                   (uchar *)table->field[0]->
+                                                   ptr,
+                                                   HA_WHOLE_KEY,
+                                                   HA_READ_KEY_EXACT))))
   {
     /* if not found, err */
     if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
@@ -675,8 +646,7 @@ int insert_server_record(TABLE *table, FOREIGN_SERVER *server)
       error= 1;
     }
     /* store each field to be inserted */
-    if ((error= store_server_fields(table, server)))
-      DBUG_RETURN(error);
+    store_server_fields(table, server);
 
     DBUG_PRINT("info",("record for server '%s' not found!",
                        server->server_name));
@@ -727,8 +697,8 @@ static int drop_server_internal(THD *thd, LEX_SERVER_OPTIONS *server_options)
   if (unlikely((error= delete_server_record_in_cache(server_options))))
     goto end;
 
-  table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT);
-  if (!table || !servers_table_is_valid(table))
+  if (unlikely(!(table= open_ltable(thd, &tables, TL_WRITE,
+                                    MYSQL_LOCK_IGNORE_TIMEOUT))))
   {
     error= my_errno;
     goto end;
@@ -855,8 +825,7 @@ int update_server(THD *thd, FOREIGN_SERVER *existing, FOREIGN_SERVER *altered)
 
   tables.init_one_table(&MYSQL_SCHEMA_NAME, &MYSQL_SERVERS_NAME, 0, TL_WRITE);
 
-  table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT);
-  if (!table || !servers_table_is_valid(table))
+  if (!(table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
   {
     error= my_errno;
     goto end;
@@ -989,7 +958,7 @@ void merge_server_struct(FOREIGN_SERVER *from, FOREIGN_SERVER *to)
     0, then index_read_idx is called to read the index to that record, the
     record then being ready to be updated, if found. If not found an error is
     set and error message printed. If the record is found, store_record is
-    called, then store_server_fields stores each field from the members of
+    called, then store_server_fields stores each field from the the members of
     the updated FOREIGN_SERVER struct.
 
   RETURN VALUE
@@ -1007,19 +976,16 @@ update_server_record(TABLE *table, FOREIGN_SERVER *server)
 
   table->use_all_columns();
   /* set the field that's the PK to the value we're looking for */
-  if (table->field[SERVER_NAME_FIELD]->store(server->server_name,
-                                             server->server_name_length,
-                                             system_charset_info))
-  {
-    DBUG_ASSERT(0); /* Protected by servers_cache */
-    THD *thd= table->in_use;
-    DBUG_ASSERT(thd->is_error());
-    return thd->get_stmt_da()->get_sql_errno();
-  }
+  table->field[0]->store(server->server_name,
+                         server->server_name_length,
+                         system_charset_info);
 
-  if ((error=table->file->ha_index_read_idx_map(table->record[0], 0,
-                  (uchar *)table->field[SERVER_NAME_FIELD]->ptr,
-                  ~(longlong)0, HA_READ_KEY_EXACT)))
+  if (unlikely((error=
+                table->file->ha_index_read_idx_map(table->record[0], 0,
+                                                   (uchar *)table->field[0]->
+                                                   ptr,
+                                                   ~(longlong)0,
+                                                   HA_READ_KEY_EXACT))))
   {
     if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
       table->file->print_error(error, MYF(0));
@@ -1030,8 +996,7 @@ update_server_record(TABLE *table, FOREIGN_SERVER *server)
   {
     /* ok, so we can update since the record exists in the table */
     store_record(table,record[1]);
-    if ((error= store_server_fields(table, server)))
-      goto end;
+    store_server_fields(table, server);
     if (unlikely((error=table->file->ha_update_row(table->record[1],
                                                    table->record[0])) &&
                  error != HA_ERR_RECORD_IS_THE_SAME))
@@ -1073,11 +1038,14 @@ delete_server_record(TABLE *table, LEX_CSTRING *name)
   table->use_all_columns();
 
   /* set the field that's the PK to the value we're looking for */
-  table->field[SERVER_NAME_FIELD]->store(name->str, name->length, system_charset_info);
+  table->field[0]->store(name->str, name->length, system_charset_info);
 
-  if ((error= table->file->ha_index_read_idx_map(table->record[0], 0,
-                (uchar *)table->field[SERVER_NAME_FIELD]->ptr,
-                HA_WHOLE_KEY, HA_READ_KEY_EXACT)))
+  if (unlikely((error=
+                table->file->ha_index_read_idx_map(table->record[0], 0,
+                                                   (uchar *)table->field[0]->
+                                                   ptr,
+                                                   HA_WHOLE_KEY,
+                                                   HA_READ_KEY_EXACT))))
   {
     if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
       table->file->print_error(error, MYF(0));
