@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2016, 2022, MariaDB Corporation.
+Copyright (c) 2016, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -33,8 +33,8 @@ Created 4/24/1996 Heikki Tuuri
 #include "dict0boot.h"
 #include "dict0crea.h"
 #include "dict0dict.h"
-#include "dict0mem.h"
 #include "dict0stats.h"
+#include "ibuf0ibuf.h"
 #include "fsp0file.h"
 #include "fts0priv.h"
 #include "mach0data.h"
@@ -168,7 +168,7 @@ name_of_col_is(
 					      dict_index_get_nth_field(
 						      index, i)));
 
-	return(strcmp(name, dict_table_get_col_name(table, tmp)) == 0);
+	return(strcmp(name, dict_table_get_col_name(table, tmp).str) == 0);
 }
 #endif /* UNIV_DEBUG */
 
@@ -215,8 +215,9 @@ dict_startscan_system(
 	mtr_t*		mtr,		/*!< in: the mini-transaction */
 	dict_table_t*	table)		/*!< in: system table */
 {
-  if (btr_pcur_open_at_index_side(true, table->indexes.start, BTR_SEARCH_LEAF,
-                                  pcur, true, 0, mtr) != DB_SUCCESS)
+  btr_pcur_init(pcur);
+  if (pcur->open_leaf(true, table->indexes.start, BTR_SEARCH_LEAF, mtr) !=
+      DB_SUCCESS)
     return nullptr;
   const rec_t *rec;
   do
@@ -864,29 +865,46 @@ err_exit:
 	return READ_OK;
 }
 
-/** Check each tablespace found in the data dictionary.
-Then look at each table defined in SYS_TABLES that has a space_id > 0
-to find all the file-per-table tablespaces.
+/** @return SELECT MAX(space) FROM sys_tables */
+static uint32_t dict_find_max_space_id(btr_pcur_t *pcur, mtr_t *mtr)
+{
+  uint32_t max_space_id= 0;
 
-In a crash recovery we already have some tablespace objects created from
-processing the REDO log. We will compare the
-space_id information in the data dictionary to what we find in the
-tablespace file. In addition, more validation will be done if recovery
-was needed and force_recovery is not set.
+  for (const rec_t *rec= dict_startscan_system(pcur, mtr, dict_sys.sys_tables);
+       rec; rec= dict_getnext_system_low(pcur, mtr))
+    if (!dict_sys_tables_rec_check(rec))
+    {
+      ulint len;
+      const byte *field=
+        rec_get_nth_field_old(rec, DICT_FLD__SYS_TABLES__SPACE, &len);
+      ut_ad(len == 4);
+      max_space_id= std::max(max_space_id, mach_read_from_4(field));
+    }
 
-We also scan the biggest space id, and store it to fil_system. */
-void dict_check_tablespaces_and_store_max_id()
+  return max_space_id;
+}
+
+/** Check MAX(SPACE) FROM SYS_TABLES and store it in fil_system.
+Open each data file if an encryption plugin has been loaded.
+
+@param spaces  set of tablespace files to open
+@param upgrade whether we need to invoke ibuf_upgrade() */
+void dict_load_tablespaces(const std::set<uint32_t> *spaces, bool upgrade)
 {
 	uint32_t	max_space_id = 0;
 	btr_pcur_t	pcur;
 	mtr_t		mtr;
 
-	DBUG_ENTER("dict_check_tablespaces_and_store_max_id");
-
 	mtr.start();
 
 	dict_sys.lock(SRW_LOCK_CALL);
 
+	if (!spaces && !upgrade
+	    && !encryption_key_id_exists(FIL_DEFAULT_ENCRYPTION_KEY)) {
+		max_space_id = dict_find_max_space_id(&pcur, &mtr);
+		goto done;
+	}
+
 	for (const rec_t *rec = dict_startscan_system(&pcur, &mtr,
 						      dict_sys.sys_tables);
 	     rec; rec = dict_getnext_system_low(&pcur, &mtr)) {
@@ -918,14 +936,6 @@ void dict_check_tablespaces_and_store_max_id()
 			continue;
 		}
 
-		if (flags2 & DICT_TF2_DISCARDED) {
-			sql_print_information("InnoDB: Ignoring tablespace"
-					      " for %.*s because "
-					      "the DISCARD flag is set",
-					      static_cast<int>(len), field);
-			continue;
-		}
-
 		/* For tables or partitions using .ibd files, the flag
 		DICT_TF2_USE_FILE_PER_TABLE was not set in MIX_LEN
 		before MySQL 5.6.5. The flag should not have been
@@ -938,6 +948,19 @@ void dict_check_tablespaces_and_store_max_id()
 			continue;
 		}
 
+		if (spaces && spaces->find(uint32_t(space_id))
+                    == spaces->end()) {
+			continue;
+		}
+
+		if (flags2 & DICT_TF2_DISCARDED) {
+			sql_print_information("InnoDB: Ignoring tablespace"
+					      " for %.*s because "
+					      "the DISCARD flag is set",
+					      static_cast<int>(len), field);
+			continue;
+		}
+
 		const span<const char> name{field, len};
 
 		char*	filepath = fil_make_filepath(nullptr, name,
@@ -970,13 +993,12 @@ void dict_check_tablespaces_and_store_max_id()
 		ut_free(filepath);
 	}
 
+done:
 	mtr.commit();
 
 	fil_set_max_space_id_if_bigger(max_space_id);
 
 	dict_sys.unlock();
-
-	DBUG_VOID_RETURN;
 }
 
 /** Error message for a delete-marked record in dict_load_column_low() */
@@ -1124,7 +1146,7 @@ err_len:
 
 			prtype = dtype_form_prtype(
 				prtype,
-				data_mysql_default_charset_coll);
+				default_charset_info->number);
 		}
 	}
 
@@ -1316,8 +1338,9 @@ static dberr_t dict_load_columns(dict_table_t *table, unsigned use_uncommitted,
 	mach_write_to_8(table_id, table->id);
 	dfield_set_data(&dfield, table_id, 8);
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
-	dberr_t err = btr_pcur_open_on_user_rec(sys_index, &tuple, PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 	if (err != DB_SUCCESS) {
 		goto func_exit;
@@ -1360,8 +1383,8 @@ static dberr_t dict_load_columns(dict_table_t *table, unsigned use_uncommitted,
 		/* Note: Currently we have one DOC_ID column that is
 		shared by all FTS indexes on a table. And only non-virtual
 		column can be used for FULLTEXT index */
-		if (innobase_strcasecmp(name,
-					FTS_DOC_ID_COL_NAME) == 0
+		if (Lex_ident_column(Lex_cstring_strlen(name)).
+		      streq(FTS_DOC_ID)
 		    && nth_v_col == ULINT_UNDEFINED) {
 			dict_col_t*	col;
 			/* As part of normal loading of tables the
@@ -1446,9 +1469,9 @@ dict_load_virtual_col(dict_table_t *table, bool uncommitted, ulint nth_v_col)
 	dfield_set_data(&dfield[1], vcol_pos, 4);
 
 	dict_index_copy_types(&tuple, sys_virtual_index, 2);
+	pcur.btr_cur.page_cur.index = sys_virtual_index;
 
-	dberr_t err = btr_pcur_open_on_user_rec(sys_virtual_index, &tuple,
-						PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 	if (err != DB_SUCCESS) {
 		goto func_exit;
@@ -1533,6 +1556,7 @@ dict_load_field_low(
 	ulint		len;
 	unsigned	pos_and_prefix_len;
 	unsigned	prefix_len;
+	bool		descending;
 	bool		first_field;
 	ulint		position;
 
@@ -1585,10 +1609,12 @@ err_len:
 	}
 
 	if (first_field || pos_and_prefix_len > 0xFFFFUL) {
-		prefix_len = pos_and_prefix_len & 0xFFFFUL;
+		prefix_len = pos_and_prefix_len & 0x7FFFUL;
+		descending = (pos_and_prefix_len & 0x8000UL);
 		position = (pos_and_prefix_len & 0xFFFF0000UL)  >> 16;
 	} else {
 		prefix_len = 0;
+		descending = false;
 		position = pos_and_prefix_len & 0xFFFFUL;
 	}
 
@@ -1638,11 +1664,12 @@ err_len:
 	if (index) {
 		dict_mem_index_add_field(
 			index, mem_heap_strdupl(heap, (const char*) field, len),
-			prefix_len);
+			prefix_len, descending);
 	} else {
 		sys_field->name = mem_heap_strdupl(
 			heap, (const char*) field, len);
 		sys_field->prefix_len = prefix_len & ((1U << 12) - 1);
+		sys_field->descending = descending;
 		*pos = position;
 	}
 
@@ -1682,9 +1709,9 @@ static dberr_t dict_load_fields(dict_index_t *index, bool uncommitted,
 	mach_write_to_8(index_id, index->id);
 	dfield_set_data(&dfield, index_id, 8);
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
-	dberr_t error = btr_pcur_open_on_user_rec(sys_index, &tuple,
-						  PAGE_CUR_GE, BTR_SEARCH_LEAF,
+	dberr_t error = btr_pcur_open_on_user_rec(&tuple, BTR_SEARCH_LEAF,
 						  &pcur, &mtr);
 	if (error != DB_SUCCESS) {
 		goto func_exit;
@@ -1940,9 +1967,9 @@ dberr_t dict_load_indexes(dict_table_t *table, bool uncommitted,
 	mach_write_to_8(table_id, table->id);
 	dfield_set_data(&dfield, table_id, 8);
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
-	dberr_t error = btr_pcur_open_on_user_rec(sys_index, &tuple,
-						  PAGE_CUR_GE, BTR_SEARCH_LEAF,
+	dberr_t error = btr_pcur_open_on_user_rec(&tuple, BTR_SEARCH_LEAF,
 						  &pcur, &mtr);
 	if (error != DB_SUCCESS) {
 		goto func_exit;
@@ -2128,7 +2155,7 @@ next_rec:
 
 	if (table->fts != NULL) {
 		dict_index_t *idx = dict_table_get_index_on_name(
-			table, FTS_DOC_ID_INDEX_NAME);
+			table, FTS_DOC_ID_INDEX.str);
 		if (idx && dict_index_is_unique(idx)) {
 			table->fts_doc_id_index = idx;
 		}
@@ -2240,22 +2267,10 @@ dict_load_tablespace(
 	/* The tablespace may already be open. */
 	table->space = fil_space_for_table_exists_in_mem(table->space_id,
 							 table->flags);
-	if (table->space) {
+	if (table->space || table->file_unreadable) {
 		return;
 	}
 
-	if (ignore_err >= DICT_ERR_IGNORE_TABLESPACE) {
-		table->file_unreadable = true;
-		return;
-	}
-
-	if (!(ignore_err & DICT_ERR_IGNORE_RECOVER_LOCK)) {
-		ib::error() << "Failed to find tablespace for table "
-			<< table->name << " in the cache. Attempting"
-			" to load the tablespace with space id "
-			<< table->space_id;
-	}
-
 	/* Use the remote filepath if needed. This parameter is optional
 	in the call to fil_ibd_open(). If not supplied, it will be built
 	from the table->name. */
@@ -2278,6 +2293,12 @@ dict_load_tablespace(
 	if (!table->space) {
 		/* We failed to find a sensible tablespace file */
 		table->file_unreadable = true;
+
+		if (!(ignore_err & DICT_ERR_IGNORE_RECOVER_LOCK)) {
+			sql_print_error("InnoDB: Failed to load tablespace "
+					ULINTPF " for table %s",
+					table->space_id, table->name);
+		}
 	}
 
 	ut_free(filepath);
@@ -2334,11 +2355,12 @@ static dict_table_t *dict_load_table_one(const span<const char> &name,
 	};
 	dfield_set_data(&dfield, name.data(), name.size());
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
 	bool uncommitted = false;
 reload:
 	mtr.start();
-	dberr_t err = btr_pcur_open_on_user_rec(sys_index, &tuple, PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 
 	if (err != DB_SUCCESS || !btr_pcur_is_on_user_rec(&pcur)) {
@@ -2468,9 +2490,7 @@ corrupted:
 				goto corrupted;
 			}
 
-			if (table->supports_instant()) {
-				err = btr_cur_instant_init(table);
-			}
+			err = btr_cur_instant_init(table);
 		}
 	} else {
 		ut_ad(ignore_err & DICT_ERR_IGNORE_INDEX);
@@ -2489,7 +2509,7 @@ corrupted:
 	if (!table->is_readable()) {
 		/* Don't attempt to load the indexes from disk. */
 	} else if (err == DB_SUCCESS) {
-		err = dict_load_foreigns(table->name.m_name, nullptr, false,
+		err = dict_load_foreigns(table->name.m_name, nullptr,
 					 0, true, ignore_err, fk_tables);
 
 		if (err != DB_SUCCESS) {
@@ -2518,7 +2538,8 @@ corrupted:
 			/* the table->fts could be created in dict_load_column
 			when a user defined FTS_DOC_ID is present, but no
 			FTS */
-			fts_free(table);
+			table->fts->~fts_t();
+			table->fts = nullptr;
 		} else if (fts_optimize_wq) {
 			fts_optimize_add_table(table);
 		} else if (table->can_be_evicted) {
@@ -2592,11 +2613,11 @@ dict_load_table_on_id(
 	mach_write_to_8(id_buf, table_id);
 	dfield_set_data(&dfield, id_buf, 8);
 	dict_index_copy_types(&tuple, sys_table_ids, 1);
+	pcur.btr_cur.page_cur.index = sys_table_ids;
 
 	dict_table_t* table = nullptr;
 
-	if (btr_pcur_open_on_user_rec(sys_table_ids, &tuple, PAGE_CUR_GE,
-				      BTR_SEARCH_LEAF, &pcur, &mtr)
+	if (btr_pcur_open_on_user_rec(&tuple, BTR_SEARCH_LEAF, &pcur, &mtr)
 	    == DB_SUCCESS
 	    && btr_pcur_is_on_user_rec(&pcur)) {
 		/*---------------------------------------------------*/
@@ -2699,9 +2720,10 @@ static dberr_t dict_load_foreign_cols(dict_foreign_t *foreign, trx_id_t trx_id)
 
 	dfield_set_data(&dfield, foreign->id, id_len);
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
 	mem_heap_t* heap = nullptr;
-	dberr_t err = btr_pcur_open_on_user_rec(sys_index, &tuple, PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 	if (err != DB_SUCCESS) {
 		goto func_exit;
@@ -2873,11 +2895,12 @@ dict_load_foreign(
 	};
 	dfield_set_data(&dfield, id.data(), id.size());
 	dict_index_copy_types(&tuple, sys_index, 1);
+	pcur.btr_cur.page_cur.index = sys_index;
 
 	mtr.start();
 
 	mem_heap_t* heap = nullptr;
-	dberr_t err = btr_pcur_open_on_user_rec(sys_index, &tuple, PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 	if (err != DB_SUCCESS) {
 		goto err_exit;
@@ -2953,7 +2976,7 @@ err_exit:
 
 	foreign->foreign_table_name = mem_heap_strdupl(
 		foreign->heap, (char*) field, len);
-	dict_mem_foreign_table_name_lookup_set(foreign, TRUE);
+	foreign->foreign_table_name_lookup_set();
 
 	const size_t foreign_table_name_len = len;
 	const size_t table_name_len = strlen(table_name);
@@ -2974,7 +2997,7 @@ err_exit:
 
 	foreign->referenced_table_name = mem_heap_strdupl(
 		foreign->heap, (const char*) field, len);
-	dict_mem_referenced_table_name_lookup_set(foreign, TRUE);
+	foreign->referenced_table_name_lookup_set();
 
 	mtr.commit();
 	if (UNIV_LIKELY_NULL(heap)) {
@@ -3039,8 +3062,6 @@ dict_load_foreigns(
 	const char*		table_name,	/*!< in: table name */
 	const char**		col_names,	/*!< in: column names, or NULL
 						to use table->col_names */
-	bool			uncommitted,	/*!< in: use READ UNCOMMITTED
-						transaction isolation level */
 	trx_id_t		trx_id,		/*!< in: DDL transaction id,
 						or 0 to check
 						recursive load of tables
@@ -3088,8 +3109,9 @@ start_load:
 	mtr.start();
 	dfield_set_data(&dfield, table_name, strlen(table_name));
 	dict_index_copy_types(&tuple, sec_index, 1);
+	pcur.btr_cur.page_cur.index = sec_index;
 
-	dberr_t err = btr_pcur_open_on_user_rec(sec_index, &tuple, PAGE_CUR_GE,
+	dberr_t err = btr_pcur_open_on_user_rec(&tuple,
 						BTR_SEARCH_LEAF, &pcur, &mtr);
 	if (err != DB_SUCCESS) {
 		DBUG_RETURN(err);
@@ -3116,12 +3138,12 @@ loop:
 	following call does the comparison in the latin1_swedish_ci
 	charset-collation, in a case-insensitive way. */
 
-	if (0 != cmp_data_data(dfield_get_type(&dfield)->mtype,
-			       dfield_get_type(&dfield)->prtype,
-			       reinterpret_cast<const byte*>(table_name),
-			       dfield_get_len(&dfield),
-			       field, len)) {
-
+	if (cmp_data(dfield_get_type(&dfield)->mtype,
+		     dfield_get_type(&dfield)->prtype,
+		     false,
+		     reinterpret_cast<const byte*>(table_name),
+		     dfield_get_len(&dfield),
+		     field, len)) {
 		goto load_next_index;
 	}
 
@@ -3155,7 +3177,7 @@ loop:
 	/* Load the foreign constraint definition to the dictionary cache */
 
 	err = len < sizeof fk_id
-		? dict_load_foreign(table_name, uncommitted, col_names, trx_id,
+		? dict_load_foreign(table_name, false, col_names, trx_id,
 				    check_recursive, check_charsets,
 				    {fk_id, len}, ignore_err, fk_tables)
 		: DB_CORRUPTION;
@@ -3190,7 +3212,6 @@ next_rec:
 
 load_next_index:
 	mtr.commit();
-	ut_free(pcur.old_rec_buf);
 
 	if ((sec_index = dict_table_get_next_index(sec_index))) {
 		/* Switch to scan index on REF_NAME, fk_max_recusive_level
@@ -3200,5 +3221,6 @@ load_next_index:
 		goto start_load;
 	}
 
+	ut_free(pcur.old_rec_buf);
 	DBUG_RETURN(DB_SUCCESS);
 }
