@@ -28,7 +28,7 @@
 #endif
 #include <m_ctype.h>
 
-static int compare_columns(const void *a, const void *b);
+static int compare_columns(MARIA_COLUMNDEF **a, MARIA_COLUMNDEF **b);
 
 
 static ulonglong update_tot_length(ulonglong tot_length, ulonglong max_rows, uint length)
@@ -1227,7 +1227,7 @@ int maria_create(const char *name, enum data_file_type datafile_type,
 	/* Enlarge files */
   DBUG_PRINT("info", ("enlarge to keystart: %lu",
                       (ulong) share.base.keystart));
-  if (mysql_file_chsize(file,(ulong) share.base.keystart,0,MYF(0)))
+  if (mysql_file_chsize(file,(ulong) share.base.keystart,0,MYF(0)) > 0)
     goto err;
 
   if (!internal_table && sync_dir && mysql_file_sync(file, MYF(0)))
@@ -1237,7 +1237,8 @@ int maria_create(const char *name, enum data_file_type datafile_type,
   {
 #ifdef USE_RELOC
     if (mysql_file_chsize(key_file_dfile, dfile,
-                          share.base.min_pack_length*ci->reloc_rows,0,MYF(0)))
+                          share.base.min_pack_length*ci->reloc_rows,0,MYF(0))
+        > 0)
       goto err;
 #endif
     if (!internal_table && sync_dir && mysql_file_sync(dfile, MYF(0)))
@@ -1335,11 +1336,9 @@ static inline int sign(long a)
 }
 
 
-static int compare_columns(const void *a_ptr_, const void *b_ptr_)
+static int compare_columns(MARIA_COLUMNDEF **a_ptr, MARIA_COLUMNDEF **b_ptr)
 {
-  const MARIA_COLUMNDEF *const *a_ptr= a_ptr_;
-  const MARIA_COLUMNDEF *const *b_ptr= b_ptr_;
-  const MARIA_COLUMNDEF *a= *a_ptr, *b= *b_ptr;
+  MARIA_COLUMNDEF *a= *a_ptr, *b= *b_ptr;
   enum en_fieldtype a_type, b_type;
 
   a_type= (a->type == FIELD_CHECK) ? FIELD_NORMAL : a->type;
